//
//  LightSensorController.m
//  LightSensor
//
//  Created by Phidgets on 2015-07-15.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "DictionaryController.h"

@implementation DictionaryController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close the example
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        PhidgetDictionary_setOnUpdateHandler(ch, NULL, NULL);
        PhidgetDictionary_setOnAddHandler(ch, NULL, NULL);
        PhidgetDictionary_setOnRemoveHandler(ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetDictionary_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
	
    PhidgetReturnCode result;
    PhidgetLog_enable(PHIDGET_LOG_INFO, nil);
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    
    if(pairs == nil)
        pairs = [[NSMutableDictionary alloc] init];
    if(sortedKeys == nil)
        sortedKeys = [[NSMutableArray alloc] init];
    tableView.delegate = self;
    tableView.dataSource = self;
    
    result = PhidgetDictionary_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDictionary_setOnUpdateHandler(ch, gotDictionaryUpdate,  (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDictionary_setOnAddHandler(ch, gotDictionaryAdd, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDictionary_setOnRemoveHandler(ch, gotDictionaryRemove, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }

    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler:)
                                           withObject:[NSValue valueWithPointer:phid]
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
static void gotDictionaryUpdate(PhidgetDictionaryHandle phid, void *context, const char *key, const char *value){
    [(__bridge id)context performSelectorOnMainThread:@selector(onUpdateHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSString stringWithUTF8String:key],[NSString stringWithUTF8String: value], nil]
                                        waitUntilDone:NO];
}
static void gotDictionaryAdd(PhidgetDictionaryHandle phid, void *context, const char *key, const char *value){
    [(__bridge id)context performSelectorOnMainThread:@selector(onUpdateHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSString stringWithUTF8String:key],[NSString stringWithUTF8String: value], nil]
                                        waitUntilDone:NO];
}
static void gotDictionaryRemove(PhidgetDictionaryHandle phid, void *context, const char *key){
    [(__bridge id)context performSelectorOnMainThread:@selector(onRemoveHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSString stringWithUTF8String:key], nil]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler:(NSValue *)phid{
    PhidgetReturnCode result;
    PhidgetDictionaryHandle dictionary = (PhidgetDictionaryHandle)[phid pointerValue];
    int serialNumber;
    const char *deviceLabel;
    
    result = Phidget_getDeviceSerialNumber((PhidgetHandle)dictionary, &serialNumber);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    
    result = Phidget_getDeviceLabel((PhidgetHandle)dictionary, &deviceLabel);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    
    [serialNumberLabel setStringValue:[NSString stringWithFormat:@"%d",serialNumber]];
    [deviceConnectedLabel setStringValue:[NSString stringWithFormat:@"%@",[NSString stringWithUTF8String:deviceLabel]]];
    pairs = [PhidgetDictionaryScanner scan:dictionary];
    [self sort];
    [tableView reloadData];
}

- (void)onDetachHandler{
    [serialNumberLabel setStringValue:@""];
    [deviceConnectedLabel setStringValue:@""];
    [deviceConnectedLabel setPlaceholderString:@"Nothing"];
}
static int errorCounter = 0;
-(void)errorHandler:(NSArray *)errorEventData{
    [self addError:[[errorEventData objectAtIndex:0] intValue]];
}

- (void)onUpdateHandler:(NSArray *)arr{
    NSString *key = [arr objectAtIndex:0];
    NSString *value = [arr objectAtIndex:1];
    [pairs setObject:value forKey:key];
    [self sort];
    [tableView reloadData];
}


- (void)onRemoveHandler:(NSArray *)arr{
    NSString *key = [arr objectAtIndex:0];
    [pairs removeObjectForKey:key];
    [self sort];
    [tableView reloadData];
}


-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

#pragma mark tableview
- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView {
    return pairs.count;
}

- (NSView *)tableView:(NSTableView *)tabView
   viewForTableColumn:(NSTableColumn *)tableColumn
                  row:(NSInteger)row {
    
    if([[tableColumn identifier]  isEqual: @"keyColumn"]){
        NSTableCellView *result = [tabView makeViewWithIdentifier:@"keyView" owner:nil];
        if(pairs.count == 0){
            result.textField.stringValue = @"";
        }
        else{
            result.textField.stringValue = sortedKeys[row];
        }
        return result;
    }
    else{
        NSTableCellView *result = [tabView makeViewWithIdentifier:@"valueView" owner:nil];
        if(pairs.count == 0){
            result.textField.stringValue = @"";
        }
        else{
            result.textField.stringValue = [NSString stringWithFormat:@"%@",[pairs valueForKey:sortedKeys[row]]];
        }
        
        return result;
    }
    
}

-(void)sort{
    [sortedKeys removeAllObjects];
    NSArray *temp = [[pairs allKeys] sortedArrayUsingSelector: @selector(compare:)];
    for (NSString *key in temp)
        [sortedKeys addObject: key];
}


#pragma mark add/remove keys
- (IBAction)addEntry:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetDictionary_add(ch, [[keyTextField stringValue] cStringUsingEncoding:NSASCIIStringEncoding], [[valueTextField stringValue] cStringUsingEncoding:NSASCIIStringEncoding]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    
    [pairs setObject:[valueTextField stringValue] forKey:[keyTextField stringValue]];
    [self sort];
    [tableView reloadData];
}

- (IBAction)removeEntry:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetDictionary_remove(ch, [sortedKeys[(int)[tableView selectedRow]] cStringUsingEncoding:NSASCIIStringEncoding]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
		
    [pairs removeObjectForKey:sortedKeys[(int)[tableView selectedRow]]];
    [self sort];
    [tableView reloadData];
}

@end
